import requests
import pandas as pd
import time
from datetime import datetime

query1 = 'https://query1.finance.yahoo.com'
query2 = 'https://query2.finance.yahoo.com'

""" Yahoo Finance V7 Multi Endpoint """
def v7multi(symbols):
    url = query2 + f"/v7/finance/quote?symbols={symbols}"
    print(url)
    req = requests.get(url)
    jsonData = req.json()
    try:
        multiData = jsonData['quoteResponse']['result']
        df = pd.DataFrame(multiData)
        df.set_index('symbol', inplace=True, drop=True)
        return df
    except Exception as e:
        errorData = jsonData['quoteResponse']['error']
        return errorData


""" Yahoo Finance V7 Options Endpoint """
def v7_options(symbol):
    url = query1 + f'/v7/finance/options/{symbol}'
    req = requests.get(url)
    jsonData = req.json()
    optionData = jsonData['optionChain']['result'][0]
    return optionData

""" Yahoo Finance V10 Single Symbol Endpoint with Module(s) feature """
def v10(symbol, module):
    url = query2 + f'/v10/finance/quoteSummary/{symbol}?modules={module}'
    req = requests.get(url)
    jsonData = req.json()
    data = jsonData['quoteSummary']['result'][0]
    return data

""" Yahoo Finance V8 Single Symbol Endpoint with start, end and interval """
def v8_period(symbol, start_date, end_date, interval):
    url = f'{query2}/v8/finance/chart/{symbol}?period1={start_date}&period2={end_date}&interval={interval}'
    return v8(url)

""" Yahoo Finance V8 Single Symbol Endpoint with range and interval """
def v8_range(symbol, range, interval):
    url = f'{query2}/v8/finance/chart/{symbol}?range={range}&interval={interval}&events=divsplit'
    return v8(url)

""" Yahoo Finance V8 """
def v8(url):
    #print(url)
    req = requests.get(url)
    jsonData = req.json()

    try:
        rangeData = jsonData['chart']['result'][0]
        # We'll need these three data points:
        timestamps = rangeData['timestamp']
        priceData = rangeData['indicators']['quote'][0]
        #events = rangeData['events']

        # Returns a list of timestamps and priceData
        return [timestamps, priceData]
    except Exception as e:
        errorData = jsonData['chart']['error']['description'].split('.')[1]
        return errorData



"""
    Yahoo Finance API Wrapper
    v0.2.91

    by Anil Sardiwal
    Started on 8th Nov'20
    last modified on 29/12/2020
"""
from .functions import getIncomeStatementHistory,  getAssetProfile,  getLivePriceData,  getMultiSymbolData, getHistoricPrices, getIncomeStatementsQtr, getBalanceSheetYearly, getBalanceSheetQtrly, getCashFlowsYearly, getCashFlowsQtrly, getFinancialAnalysisData, getMajorHolders, getOptionsData

# Symbol class object
class Symbol:
    def __init__(self,  symbol):
        self.symbol = symbol.upper()

    def incomeStatements(self):
        return getIncomeStatementHistory(self.symbol)

    def incomeStatementsQtr(self):
        return getIncomeStatementsQtr(self.symbol)

    def balanceSheets(self):
        return getBalanceSheetYearly(self.symbol)

    def balanceSheetsQtr(self):
        return getBalanceSheetQtrly(self.symbol)

    def cashFlows(self):
        return getCashFlowsYearly(self.symbol)

    def cashFlowsQtr(self):
        return getCashFlowsQtrly(self.symbol)

    def analysis(self):
        return getFinancialAnalysisData(self.symbol)

    def shareholding(self):
        return getMajorHolders(self.symbol)

    def profile(self, kmp=''):
        return getAssetProfile(self.symbol, kmp)

    def livePriceData(self):
        return getLivePriceData(self.symbol)

    def multi(self):
        return getMultiSymbolData(self.symbol)

    def history(self, start=None, end=None, period='max', interval='1d'):
        return getHistoricPrices(self.symbol, start, end, period, interval)

    def options(self, dataType='calls'):
        return getOptionsData(self.symbol, dataType)

# Test!
if __name__ == "__main__":
    tsla = Symbol('TSLA')
    multi = Symbol('TSLA, AAPL, GOOG, AMZN')
    #msft = Symbol('MSFT')
    #print(msft.profile())

    #msft = Symbol('TSLA', start='2020-12-01', end='2020-12-05')
    #msft = Symbol('TSLA', start='2020-12-01', end='2020-12-02', interval='3m')
    msft = Symbol('RCF.NS')
    print(msft.history())
    #msft.history().to_csv('okok.csv')

    #symbols = Symbol('TSLA, MSFT, AAPL, GOOG')
    #print(symbols.multi()['marketCap'])

    #tsla = Symbol('TSLA')
    #print(tsla.incomeStatementsQtr())

    #tsla = Symbol('TSLA')
    #print(tsla.balanceSheets())

    #tsla = Symbol('TSLA')
    #print(tsla.cashFlowsQtr())


    #print(tsla.analysis())
    #print(tsla.shareholding())
    #tsla.livePriceData().to_csv('okokok.csv')
    #print(tsla.livePriceData())

    #print(tsla.options('quotessd'))
    #tsla.options('quotes').to_csv('okokok.csv')

    #print(tsla.profile())
    #tsla.profile('kmp').to_csv("okok.csv")

    #print(multi.multi())



import pandas as pd
import time

from .utils import chunk_list, epochToDatetimeList, returnDf, formatColumns
from .engines import v8_period, v8_range, v7multi, v10, v7_options

""" Gets a company's asset profile as in: address, summary, website, employees, etc. """
def getAssetProfile(symbol, kmp):
    asset_profile = v10(symbol, 'assetProfile')['assetProfile']

    # If user asks for kmp (key managerial personnel), return specific data
    if kmp == 'kmp':
        profile = asset_profile['companyOfficers']
        df = pd.DataFrame(profile)

        # formatColumns(df) because some values have options for 'raw' and 'fmt'.
        return formatColumns(df)
    else:
        df = pd.DataFrame(asset_profile)
        # Keep only the first row i.e. 'raw'
        df = df.iloc[:1]
        return df

# To be modified
def getLivePriceData(symbol):
    price_data = v10(symbol, 'price')['price']
    df = pd.DataFrame(price_data)

    # Keep only the first row i.e. 'raw'
    df = df.iloc[:1]

    return df

"""
    This function returns data for multiple symbols in one go
    (or more if symbols are more than 100).

    Input: "AAPL, MSFT, ESCORTS.NS, JSLHISAR.NS" #Spaces_are_optional

    Outputs: CMP, marketCap, Day High+Low+Volume, bid_ask, and more!
"""
def getMultiSymbolData(symbols):
    # A base DataFrame to which we will append more DFs later in this function.
    base_df = pd.DataFrame()

    # first remove spaces, if any
    symbols = symbols.replace(" ", "")

    # Extract every symbol separated by comma and store in a set (unique values)
    symbolss = symbols.split(',')
    symbolSet = set()
    for symbol in symbolss:
        symbolSet.add(symbol)

    symbolSet = sorted(symbolSet) # sort the list

    # if symbolSet length is  more than 100, split it into multiples of 100, and
    # find values and attach to one DataFrame
    if len(symbolSet) > 100:
        # Splitting a list into multiple lists of size 100
        symbolList = list(chunk_list(symbolSet, 100))

        # Iterate through each list containing
        for singleSymbolList in symbolList:
            # convert list into a single String
            symbols = ",".join(str(x) for x in singleSymbolList)

            # Run v7multi and get a DataFrame and append to base_df
            base_df = base_df.append(v7multi(symbols))
    else:
        # convert list into a single String
        symbols = ",".join(str(x) for x in symbolSet)

        # Run v7multi and get a DataFrame and append to base_df
        base_df = base_df.append(v7multi(symbols))
    return base_df

"""
    It gets symbol's historic price data (open, high, low, close) with timestamps.

    Start and End are provided as "YY-mm-dd" and converted to epocs (Yahoo Finance)

    It then calls v8 function (written in this module) and we convert timestamps
    and priceData into separate DataFrames and then we join both DFs and return.
"""
def getHistoricPrices(symbol, start_date=None, end_date=None, period=None, interval=None):
    data_lists = []
    if start_date and end_date:
        start = int(time.mktime(time.strptime(str(start_date), '%Y-%m-%d')))
        end = int(time.mktime(time.strptime(str(end_date), '%Y-%m-%d'))) + 86400    #end is always +1day

        #print(f'start: {start}, end: {end}')

        data_lists = v8_period(symbol, start, end, interval)
    else:
        data_lists = v8_range(symbol, period, interval)

    try:
        timestamps = pd.DataFrame(epochToDatetimeList(data_lists[0]))
        priceData = pd.DataFrame(data_lists[1], columns=['open', 'high', 'low', 'close', 'volume'])

        # Join both DFs
        final_df = timestamps.join(priceData)

        final_df.columns = [['Datetime', 'Open', 'High', 'Low', 'Close', 'Volume']]
        return final_df
    except Exception as e:
        return data_lists

"""This function takes in a symbol and gets the latest Income Statements from Yahoo Finance"""
def getIncomeStatementHistory(symbol):
    income_statements = v10(symbol, 'incomeStatementHistory')['incomeStatementHistory']['incomeStatementHistory']
    return returnDf(income_statements)

"""This function takes in a symbol and gets the latest Income Statements (Qtrly) from Yahoo Finance"""
def getIncomeStatementsQtr(symbol):
    income_statements = v10(symbol, 'incomeStatementHistoryQuarterly')['incomeStatementHistoryQuarterly']['incomeStatementHistory']
    return returnDf(income_statements)

def getBalanceSheetYearly(symbol):
    bs = v10(symbol, 'balanceSheetHistory')['balanceSheetHistory']['balanceSheetStatements']
    return returnDf(bs)

def getBalanceSheetQtrly(symbol):
    bs = v10(symbol, 'balanceSheetHistoryQuarterly')['balanceSheetHistoryQuarterly']['balanceSheetStatements']
    return returnDf(bs)

def getCashFlowsYearly(symbol):
    cf = v10(symbol, 'cashflowStatementHistory')['cashflowStatementHistory']['cashflowStatements']
    return returnDf(cf)

def getCashFlowsQtrly(symbol):
    cf = v10(symbol, 'cashflowStatementHistoryQuarterly')['cashflowStatementHistoryQuarterly']['cashflowStatements']
    return returnDf(cf)

def getFinancialAnalysisData(symbol):
    analysisData = v10(symbol, 'financialData')['financialData']
    df = pd.DataFrame(analysisData)

    # Keep only the first row i.e. 'raw'
    df = df.iloc[:1]
    return df

"""
    Returns Major Shareholders' percentage share by company's Symbol.
    New: Added retailers' shareholding.
"""
def getMajorHolders(symbol):
    holders = v10(symbol, 'majorHoldersBreakdown')['majorHoldersBreakdown']
    df = pd.DataFrame(holders)

    # Keep only the first row i.e. 'raw'
    df = df.iloc[:1]

    # renaming columns
    df = df.rename(columns={"insidersPercentHeld": "promoters", "institutionsFloatPercentHeld": "institutions"})

    pd.to_numeric(df['promoters'])
    pd.to_numeric(df['institutions'])

    # creating a new column: 'retailers': {1 - (promoters + institutions)}
    df['retailers'] =  1 - (df['promoters'] + df['institutions'])
    df = df[['promoters', 'institutions', 'retailers', 'institutionsCount']]

    return df

"""
    This function gets options data of a given symbol. Probably only available for US stocks.
    Also takes in an input of dataType i.e. calls, puts, dates, strikes, quotes
"""
def getOptionsData(symbol, dataType):
    optionsData = v7_options(symbol)
    options = None

    if dataType == 'calls':
        options = optionsData['options'][0]['calls']
    elif dataType == 'puts':
        options = optionsData['options'][0]['puts']
    elif dataType == 'dates':
        options = optionsData['expirationDates']
        options = epochToDatetimeList(options)  #because timestamps are unreadable in raw format
    elif dataType == 'strikes':
        options = optionsData['strikes']
    elif dataType == 'quotes':
        options = optionsData['quote']
        # Passes scaler values, so had to pass an index and return function from here.
        df = pd.DataFrame(options, index=[0])
        return df
    else:
        # Wrong dataType, empty dataFrame is returned.
        return pd.DataFrame()

    df = pd.DataFrame(options)
    return df



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="yahfin", # Replace with your own username
    version="0.2.93",
    author="Anil Sardiwal",
    author_email="theonlyanil@gmail.com",
    description="Yahoo Finance Python Wrapper (Unofficial)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/theonlyanil/yahfin",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Financial and Insurance Industry",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    python_requires='>=3.7.7',
)
